import java.awt.*;
import java.awt.event.*;
import com.jinsight.jetchart.*;

import com.jinsight.svg.*; // The SVG encoder is in the svg package.
import java.io.*; 

/**
 * SVG  stands for  Scalable  Vector Graphics, and  it is a language for  describing
 * two dimensional vector graphics using XML. SVG  content  is  saved  into  a plain 
 * text  file  with the '.svg'  extension, therefore it can be easily  changed using 
 * any word processor.
 * The  use  of SVG is  rapidly  increasing,  especially  because  of the quality of
 * vectorial  graphics, superseding  rasterized  image formats such as GIF, JPEG and 
 * PNG.
 * JetChart  implements  an  SVG  encoder  to  output  chart images as SVG text. SVG
 * encoding  consists  basically  of  passing a  reference  of a GenericGraph object
 * to a  ChartEncoder  instance and triggering  the encoding process by invoking the
 * method ChartEncoder.svgEncode(OutputStream out,boolean encodeAsImage,int quality).
 * The first  argument  can be  any OutputStream  object which is used to output the
 * SVG text. This example outputs data to an external file called 'chart.svg'.
 * The second argument specifies whether the chart image shall be encoded into a PNG
 * image and  outputted  as a base64 sequence of characters or shall be encoded into
 * a sequence of SVG  instructions  corresponding to each one of the drawing methods
 * invoked.
 * The  third  argument is an  indicator  to the SVG viewer as to the quality of the
 * image to be displayed.
 *
 * There are several SVG viewers available,which can be used to load and display the 
 * SVG file generated by this application. The Adobe SVG Viewer is probably the most
 * used and it can be found at the following address:
 * http://www.adobe.com/svg/viewer/install/main.html
 * There is also an SVG viewer called Batik, which is written in Java and can be found
 * at http://xm.apache.org/batik .
 *
 * This is an AWT example, using heavyweight components to display charts.
 *
 * More  in-depth  information  about  Scalable  Vector  Graphics can  be  found  at 
 * the address http://www.w3c.org/TR/svg.
 *
 * @author Andre de Lima Soares - alsoares@jinsight.com
 */

public class Main extends Frame implements ItemListener,ActionListener {

    GenericGraph currentGraph;

    Panel p;
    
    String[] labels;
    double[] values1,values2,values3;

    double[][] values4,values5,values6,values7;
    
    CardLayout cardLayout;
    
    Button svgButton;
    
    public Main() {
	
	// Creates radio buttons to select different types of graphs.
        CheckboxGroup cbg=new CheckboxGroup();
	Checkbox cb1=new Checkbox("Line",cbg,true);
        cb1.addItemListener(this);
	Checkbox cb2=new Checkbox("Bar",cbg,false);
        cb2.addItemListener(this);
	Checkbox cb3=new Checkbox("Stacked bars",cbg,false);
        cb3.addItemListener(this); 
        Checkbox cb4=new Checkbox("Area",cbg,false);
        cb4.addItemListener(this);
        Checkbox cb5=new Checkbox("Image",cbg,false);
        cb5.addItemListener(this);
	Checkbox cb6=new Checkbox("Point",cbg,false);
	cb6.addItemListener(this);
	Checkbox cb7=new Checkbox("OHLC",cbg,false);
	cb7.addItemListener(this);
	Checkbox cb8=new Checkbox("Pie",cbg,false);
	cb8.addItemListener(this);
	Checkbox cb9=new Checkbox("XY",cbg,false);
	cb9.addItemListener(this);
	Checkbox cb10=new Checkbox("Bubble",cbg,false);
	cb10.addItemListener(this);

	// Creates a top panel and adds the radio buttons to it.
        Panel topPanel=new Panel(new FlowLayout(FlowLayout.LEFT));
        topPanel.add(cb1);
        topPanel.add(cb2);
        topPanel.add(cb3);
        topPanel.add(cb4);
        topPanel.add(cb5);
	topPanel.add(cb6);
	topPanel.add(cb7);
	topPanel.add(cb8);
	topPanel.add(cb9);
	topPanel.add(cb10);

        add("North",topPanel);

	// Creates a panel to display the graphs, setting its layout manager
	// to a CardLayout.
	cardLayout=new CardLayout();

	p=new Panel(cardLayout);

	// Creates a bottom panel to display the SVG button.
	Panel bottomPanel=new Panel();
	svgButton=new Button("Generate SVG");
	svgButton.addActionListener(this);
	bottomPanel.add(svgButton);

	add("South",bottomPanel);

	// Specifies labels and values for the different types of graphs.
	createData();
	
	// Creates different types of graphs.
	createLineGraph();
	createBarGraph();
        createStackedBarsGraph();
        createAreaGraph();
	createImageGraph();
	createPointGraph();
	createOHLCGraph();
	createPieGraph();
	createXYGraph();
	createBubbleGraph();

	add("Center",p);

	// Window adapter to exit application.
	addWindowListener(new WindowAdapter() {
		public void windowClosing(WindowEvent evt) {
		    System.exit(0);
		}
	    });
			  

	setSize(700,450);

	setVisible(true);

    }

    // The SVG encoding process is started in this event handler.
    public void actionPerformed(ActionEvent evt) {
        
	// Gets all components added to the main panel to check which is
	// the visible one.
	Component[] components=p.getComponents();
	
	// Starts loop across the array to get the visible graph.
	for (int counter=0;counter<components.length;counter++) {
	    
	    if (components[counter].isVisible()) {
		
		GenericGraph graph=(GenericGraph)components[counter];
		
		// The SVG encoding process starts here, outputting the SVG code into
		// a file called 'chart.svg'.
		FileOutputStream out=null;
		try {
		    ChartEncoder ce=new ChartEncoder(graph);
		    File f=new File("chart.svg");
		    out=new FileOutputStream(f);

		    /**
		     * This is the method that actually starts the SVG encoding process.
		     * The second argument of svgEncode indicates whether the graph image
		     * has to be encoded as a single  image, in which  case a base64 encoded
		     * string is generated on a PNG image, or a sequence of SVG instructions
		     * corresponding to each of the Graphics class drawing methods is outputted.
		     * The second argument is an indicator to the SVG encoder that the encoded
		     * image has to be displayed in high or low quality.
		     */
		    ce.svgEncode(out,false,SVGEncoder.HIGH_QUALITY);
		    out.flush();
		}
		catch (IOException e) {
		    e.printStackTrace();
		}
		finally {
		    try {
			if (out!=null)
			    out.close();
		    }
		    catch (IOException e) {
			e.printStackTrace();
		    }
		}
		
		break;
		
	    }
	}
	 
    }

    // Captures a click on each of the radio buttons and display respective
    // graph.
    public void itemStateChanged(ItemEvent evt) {
       Checkbox cb=(Checkbox)evt.getSource();
       String label=cb.getLabel();
       
       cardLayout.show(p,label);

    }

    // Creates data to be populated by the graphs.
    private void createData() {
	
	labels=new String[]{"label1","label2","label3","label4","label5","label6","label7"};
	values1=new double[]{100,130,80,90,60,110,55};
        values2=new double[]{50,-90,40,75,-65,130,100};
        values3=new double[]{72,91,44,-67,100,-40,80};
	
	values4=new double[][]{{185.00,178.00,179.00,182.50},
			       {196.00,183.50,183.95,192.90},
			       {196.95,189.20,194.00,190.00},
			       {195.50,186.55,191.00,187.25},
			       {195.60,190.00,192.00,192.00},
			       {207.35,190.10,191.10,207.35},
			       {214.00,193.10,211.25,196.15}};
	
	values5=new double[][]{{131.90,123.50,129.00,124.30},
			       {132.80,128.00,130.00,128.45},
			       {130.50,122.55,127.00,129.60},
			       {131.00,126.50,131.00,127.50},
			       {136.75,129.00,135.00,131.30},
			       {135.00,124.25,129.50,133.95},
			       {131.50,126.50,128.00,129.65}};
	
	values6=new double[][]{{35.2,10.1},{37.3,10.85},{44.9,12.48},{17.68,12.63},
			       {63.55,24.58},{103.8,25.6},{250.2,39.7},{81.98,43.1},
			       {126.1,48.4},{85.5,74.6}};
    
	values7=new double[][]{{100,80,30},{50,-120,40},{30,-40,90},
				{70,-100,60},{40,-50,50},{-65,80,70},
				{-40,30,50},{-30,-80,40},{-60,-90,50},
				{-20,-60,70}};

    }

    // Creates a line graph.
    private void createLineGraph() {
	
	LineSerie ls=new LineSerie(values1,"Line Series");
	ls.setColor(Color.red);
	ls.setThickness(2);

        GraphSerie[] gs={ls};
       
	Graph graph=createGraph(gs,false,false,Color.white,Color.cyan);

	p.add(graph,"Line");

    }
			
    // Creates a bar graph.
    private void createBarGraph() {

	BarSerie bs=new BarSerie(values1,"Bar Series");
	bs.setColor(Color.yellow);

        GraphSerie[] gs={bs};

	Graph graph=createGraph(gs,true,false,Color.white,Color.blue);
	
	p.add(graph,"Bar");

    }

    // Creates a stacked bars graph.
    private void createStackedBarsGraph() {

        StackBarSerie sb1=new StackBarSerie(values1,"Stacked bar series1");
        StackBarSerie sb2=new StackBarSerie(values2,"Stacked bar series2");
        StackBarSerie sb3=new StackBarSerie(values3,"Stacked bar series3");

        StackBarSerie.setStackBarWidth(20);

        sb1.setColor(Color.green.darker());
	sb1.setMarkLegendEnabled(true);
	sb1.setMarkLegendPosition(StackBarSerie.INSIDE);

        sb2.setColor(Color.red);
	sb2.setMarkLegendEnabled(true);
	sb2.setMarkLegendPosition(StackBarSerie.INSIDE);

        sb3.setColor(Color.blue);
	sb3.setMarkLegendEnabled(true);
	sb3.setMarkLegendPosition(StackBarSerie.INSIDE);

        GraphSerie[] gs={sb1,sb2,sb3};

        Graph graph=createGraph(gs,true,true,Color.white,new Color(0,128,255));

        p.add(graph,"Stacked bars");

    }

    // Creates an area graph.
    private void createAreaGraph() {

	AreaSerie as=new AreaSerie(values2,"Area Series");
	as.setColor(Color.pink);

        GraphSerie[] gs={as};
       
	Graph graph=createGraph(gs,true,false,Color.white,Color.cyan);

	p.add(graph,"Area");

    }

    // Creates an image graph.
    private void createImageGraph() {
	
	Image img1=Toolkit.getDefaultToolkit().getImage("image1.gif");
	Image img2=Toolkit.getDefaultToolkit().getImage("image2.gif");
	Image img3=Toolkit.getDefaultToolkit().getImage("image3.gif");
	Image img4=Toolkit.getDefaultToolkit().getImage("image4.gif");
	Image img5=Toolkit.getDefaultToolkit().getImage("image5.gif");
	Image img6=Toolkit.getDefaultToolkit().getImage("image6.gif");
	
	Image[] img={img1,img2,img3,img4,img5,img6};

	ImageSerie is=new ImageSerie(values1,img);

	GraphSerie[] gs={is};

	Graph graph=createGraph(gs,false,false,Color.white,Color.white);

	p.add(graph,"Image");

    }
    
    // Creates a point graph.
    private void createPointGraph() {
	
	PointSerie ps1=new PointSerie(values2,"Point Series 1");
	ps1.setColor(Color.red);
	
	PointSerie ps2=new PointSerie(values1,"Point Series 2");
	ps2.setColor(Color.yellow);
	ps2.setShape(PointSerie.TRIANGLE);

        GraphSerie[] gs={ps1,ps2};
	
	Graph graph=createGraph(gs,true,false,new Color(0,128,255),Color.white);
	
	p.add(graph,"Point");
	
    }

    // Creates an OHLC graph.
    private void createOHLCGraph() {
	
	CandleStickSerie cs=new CandleStickSerie(values4,"CandleStick series");
	cs.setWidth(10);

	OHLCBarSerie obs=new OHLCBarSerie(values5,"OHLC Bar series");
	obs.setBullishColor(Color.blue);
	obs.setBearishColor(Color.red);

	GraphSerie[] gs={cs,obs};

	Graph graph=createGraph(gs,false,false,Color.white,Color.white);

	graph.setLegendEnabled(false);

	Scale scale=graph.getGraphSet(0).getScale();
	scale.setAutoScaleEnabled(false);
	scale.setMaxValue(230);
	scale.setMinValue(100);
	scale.setIncrement(30);

	p.add(graph,"OHLC");

    }

    // Creates a pie graph.
    private void createPieGraph() {
	
	PieGraph graph=new PieGraph();
	graph.setTitle(new String[]{"The JetChart Library","SVG Encoding Demo"});
	graph.set3DEnabled(true);
	graph.getToolTip().setEnabled(true);

	PieSerie ps=new PieSerie();
	ps.setXRadiusInset(40);
	ps.setYRadiusInset(20);

	Color[] colors={Color.red,Color.yellow,Color.green,Color.blue,Color.cyan,Color.magenta,Color.pink};

	for (int counter=0;counter<values1.length;counter++) {
	    Slice slice=new Slice();
	    slice.setTitle(labels[counter]);
	    slice.setValue(values1[counter]);
	    slice.setColor(colors[counter]);
	    slice.setPosition(10);
	    slice.getSliceLegend().setEnabled(true);
	    ps.addSlice(slice);
	}
	
	graph.addSerie(ps);

	p.add(graph,"Pie");

    }

    // Creates an XY graph.
    private void createXYGraph() {

	ScatterGraph graph=new ScatterGraph();
	graph.setTitle(new String[]{"The JetChart Library","SVG Encoding Demo"});
	graph.getToolTip().setEnabled(true);
	
	Grid grid=graph.getGrid();
	grid.setEnabled(true);
	grid.setStyle(Grid.DASHED);
	grid.setColor(Color.lightGray);

	GridCrossedLines gcl=graph.getGridCrossedLines();
	gcl.setEnabled(true);
	gcl.setStyle(Grid.DASHED);
	gcl.setColor(Color.lightGray);

	XYSerie xy=new XYSerie(values6,"XY series");
	xy.setColor(Color.blue);
	
	graph.addSerie(xy);

	p.add(graph,"XY");

    }
    
    // Creates a bubble graph.
    private void createBubbleGraph() {

	ScatterGraph graph=new ScatterGraph();
	graph.setTitle(new String[]{"The JetChart Library","SVG Encoding Demo"});
	graph.getToolTip().setEnabled(true);
	
	Image backgroundImage=Toolkit.getDefaultToolkit().getImage("ice.jpg");
	
	graph.setBackgroundImage(backgroundImage,Graph.TILE);

	Scale vScale=graph.getVScale();
	vScale.setAutoScaleEnabled(false);
	vScale.setMaxValue(150);
	vScale.setMinValue(-150);
	vScale.setIncrement(30);
	vScale.setPosition(Scale.LEFT_RIGHT);
	vScale.setValueFormat("###,###");
	
	Scale hScale=graph.getHScale();
	hScale.setAutoScaleEnabled(false);
	hScale.setMaxValue(120);
	hScale.setMinValue(-120);
	hScale.setIncrement(30);
	hScale.setPosition(Scale.TOP_BOTTOM);
	hScale.setValueFormat("###,###");

	Grid grid=graph.getGrid();
	grid.setEnabled(true);
	grid.setStyle(Grid.DASHED);
	grid.setColor(Color.gray);

	GridCrossedLines gcl=graph.getGridCrossedLines();
	gcl.setEnabled(true);
	gcl.setStyle(Grid.DASHED);
	gcl.setColor(Color.gray);

	BubbleSerie bs=new BubbleSerie(values7,"Bubble series");
	bs.setColor(Color.cyan);
	bs.setBorderEnabled(true);

	graph.addSerie(bs);

	p.add(graph,"Bubble");

    }

    // This method creates the chart context for the line,bar,stacked bars,area,point,image and ohlc charts.
    private Graph createGraph(GraphSerie[] gs,boolean is3DEnabled,boolean isHor,Color gradColor1,Color gradColor2) {

	Graph graph=new Graph();

	graph.setLabels(labels);

	graph.setTitle(new String[]{"The JetChart Library","SVG Encoding Demo"});
	graph.setGradientColors(gradColor1,gradColor2);

	graph.getToolTip().setEnabled(true);

	Grid grid=graph.getGraphSet(0).getGrid();
	grid.setEnabled(true);
	grid.setColor(Color.lightGray);
		
	XAxis xAxis=graph.getGraphSet(0).getXAxis();
	xAxis.set3DColor(Color.orange);

	YAxis yAxis=graph.getGraphSet(0).getYAxis();
	yAxis.set3DColor(Color.orange);

        for (int counter=0;counter<gs.length;counter++) 
	    graph.addSerie(gs[counter]);

	graph.set3DEnabled(is3DEnabled);

        graph.setHorizontalGraphEnabled(isHor);

	return graph;

    }
    
    public static void main(String[] args) {
	new Main();
    }

}
